#! /bin/bash
# ----------------------------------------------------------------------------
# buildPackage.sh - Build the package on the current folder
#
# Jenkins environment variables must be set for proper functionality!
# Build job name must follow the form:
# <something>__<package-name>__<alpine-release>_<arch>
#
# Creation   :  2013-11-09  starwarsfan
#
# Copyright (c) 2013-2018 the eisfair team, team(at)eisfair(dot)org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# ----------------------------------------------------------------------------

#exec 2> /tmp/buildPackage-trace$$.txt
#set -x

rtc=0



helpMe ()
{
    echo "

    Usage:
    ${0} [options]
        This script builds the package which is determined out of environment
        variable JOB_NAME and stores the result on the appropriate folder
        given by env var CI_RESULTFOLDER_EISFAIR_NG. These nvironment
        variables might be set using options described below. Otherwise the
        script will fail.
        Script should be executed out of repository root.

    Optional parameters:
    -a <alpine-version>
    -j <job-name>
        Set variable JOB_NAME. Value must follow the form
        'eisfair-ng/<alpine-release>/<stage>/<architecture>/<package>'
        so i. e. 'eisfair-ng/v3.1/testing/x86_64/bonnie'.
        Normally it is set by Jenkins as an env var.
    -r <result-folder>
        Set variable CI_RESULTFOLDER_EISFAIR_NG.
    -h  Show this help

    "
}



checkEnvironment ()
{
    info "##### Checking environment:"
    if [ -z "${JOB_NAME}" ] ; then
        helpMe
        die 100 "ERROR: Env var 'JOB_NAME' must be set or option -j must be used!"
    fi
    if [ -z "${CI_RESULTFOLDER_EISFAIR_NG}" ] ; then
        helpMe
        die 101 "ERROR: Env var 'CI_RESULTFOLDER_EISFAIR_NG' must be set or option -r must be used!"
    fi
    info "Done"
}



extractVariables ()
{
    # Extract package name, stage and arch from eisfair-ng/<alpine-release>/<stage>/<architecture>/<package>
    # Example: JOB_NAME='eisfair-ng/v3.1/testing/x86_64/bonnie'
    packageName=${JOB_NAME##*\/}
    packageArch=${JOB_NAME%\/*}
    packageArch=${packageArch#*\/}
    packageStage=${packageArch%\/*}
    packageStage=${packageStage#*\/}
    packageArch=${packageArch##*\/}
    if [ -z "${ALPINE_VERSION}" ] ; then
        alpineRelease=${packageStage%\/*}
    else
        alpineRelease=${ALPINE_VERSION}
    fi
}



buildPackage ()
{
    info "Updating pkg repository"
    sudo apk update

    info "Cd to ${packageName}"
    cd ${packageName}

    info "Removing previously build apk files"
    rm -f *.apk

#    echo "Updating checksums"
#    abuild checksum

    info "Building package"
    abuild -r -c
    rtc=$?
    if [ "${rtc}" = 0 ] ; then
        info "Copying apk file(s) to repository folder"
        checkDeploymentDestination
        if ls *.apk >/dev/null 2>&1 ; then
            cp -f *.apk ${CI_RESULTFOLDER_EISFAIR_NG}/${alpineRelease}/${packageStage}/${packageArch}/
            rtc=$?
        else
            cp -f ~/packages/${packageName}/${packageArch}/*.apk ${CI_RESULTFOLDER_EISFAIR_NG}/${alpineRelease}/${packageStage}/${packageArch}/
            rtc=$?
        fi
    else
        exit ${rtc}
    fi
}



checkDeploymentDestination ()
{
    if [ ! -d ${CI_RESULTFOLDER_EISFAIR_NG}/${alpineRelease}/${packageStage}/${packageArch} ] ; then
        info "Repository directory not existing, creating it"
        mkdir -p ${CI_RESULTFOLDER_EISFAIR_NG}/${alpineRelease}/${packageStage}/${packageArch}
    fi
}



######### HELPER-FUNCTIONS #########
_init() {
    if [ -n "${TERM}" -a "${TERM}" != "dumb" ]; then
        GREEN=$(tput setaf 2) RED=$(tput setaf 1) BLUE="$(tput setaf 4)"
        LTGREYBG="$(tput setab 7)"
        NORMAL=$(tput sgr0) BLINK=$(tput blink)
    else
        GREEN="" RED="" BLUE="" LTGREYBG="" NORMAL="" BLINK=""
    fi
}
die() {
    error=${1:-1}
    shift
    error "$*" >&2
    exit ${error}
}
info() {
    printf "${NORMAL}%-7s: %s${NORMAL}\n" "Info" "$*"
}
error() {
    printf "${RED}%-7s: %s${NORMAL}\n" "Error" "$*"
}
warning() {
    printf "${BLUE}%-7s: %s${NORMAL}\n" "Warning" "$*"
}

######### HANDLE OPTIONS, CALL MAIN #########
_init
ALPINE_VERSION=''

while getopts a:j:r:h? option; do
    case ${option} in
        a) ALPINE_VERSION="${OPTARG}";;
        j) JOB_NAME="${OPTARG}";;
        r) CI_RESULTFOLDER_EISFAIR_NG="${OPTARG}";;
        h|?) helpMe && exit 0;;
        *) die 90 "invalid option \"${OPTARG}\"";;
    esac
done

checkEnvironment
extractVariables
buildPackage

exit ${rtc}

# ============================================================================
# End
# ============================================================================
